/*
 * Copyright (c) 2010, Oracle and/or its affiliates. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; version 2 of the
 * License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301  USA
 */

#ifndef _STARTER_POPUP_H_
#define _STARTER_POPUP_H_

#include "mforms/popup.h"

#include "wb_context.h"

namespace wb {

  typedef struct
  {
    MySQL::Geometry::Rect bounds;       // Overall bounds of the entry.
    MySQL::Geometry::Rect icon_bounds;  // Position and size of the icon relative to overall bounds.
  } PopupEntry;

  class StarterPopup : public mforms::Popup
  {
  private:
    WBContext* _context;
    std::vector<PopupEntry> _entries;
    cairo_surface_t* _mysql_plugin_overlay;
    cairo_surface_t* _community_plugin_overlay;
    cairo_surface_t* _legal_image;

    cairo_surface_t* _button_parts[3];
    MySQL::Geometry::Rect _more_button_bounds;
    MySQL::Geometry::Rect _back_button_bounds;
    MySQL::Geometry::Rect _next_button_bounds;
    cairo_pattern_t* _button_pattern;
    int _button_inner_width;                 // Precomputed width for the tile pattern.
  protected:
    void draw_button(cairo_t* cr, double x, double y, double w, double h);
    void draw_entry(cairo_t* cr, app_StarterRef starter, int x, int y, int w, int h);
  public:
    StarterPopup(WBContext* context);
    ~StarterPopup();

    virtual void repaint(cairo_t *cr, int x, int y, int w, int h);

    virtual void mouse_down(int button, int x, int y);
    virtual void mouse_up(int button, int x, int y);
    virtual void mouse_click(int button, int x, int y);
    virtual void mouse_enter();
    virtual void mouse_leave();
    virtual void mouse_move(int x, int y);
  };
};

#endif // _STARTER_POPUP_H_
